@@ -12,21 +12,19 @@ from mkosi import run_verb
 from mkosi.config import parse_config
 from mkosi.log import log_setup
 from mkosi.run import find_binary, run, uncaught_exception_handler
-from mkosi.user import INVOKING_USER
 from mkosi.util import resource_path
 
 
-def onsigterm(signal: int, frame: Optional[FrameType]) -> None:
+def onsignal(signal: int, frame: Optional[FrameType]) -> None:
     raise KeyboardInterrupt()
 
 
 @uncaught_exception_handler()
 def main() -> None:
-    signal.signal(signal.SIGTERM, onsigterm)
+    signal.signal(signal.SIGTERM, onsignal)
+    signal.signal(signal.SIGHUP, onsignal)
 
     log_setup()
-    # Ensure that the name and home of the user we are running as are resolved as early as possible.
-    INVOKING_USER.init()
 
     with resource_path(mkosi.resources) as resources:
         args, images = parse_config(sys.argv[1:], resources=resources)
